
#include <qcstring.h>
#include <qsocket.h>
#include <qdatetime.h>
#include <qbitarray.h>

#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include <kapplication.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <kinstance.h>
#include <kglobal.h>
#include <kstandarddirs.h>
#include <klocale.h>
#include <kurl.h>
#include <ksock.h>

#include <sys/stat.h>
#include <qfile.h>
#include <qdir.h>

#include "kiobashin.h"

#define BUFFERSIZE 254
using namespace KIO;


kio_kiobashinProtocol::kio_kiobashinProtocol(const QCString &pool_socket, const QCString &app_socket)
    : SlaveBase("kio_kiobashin", pool_socket, app_socket)
{
    kdDebug() << "kio_kiobashinProtocol::kio_kiobashinProtocol()" << endl;
}


kio_kiobashinProtocol::~kio_kiobashinProtocol()
{
    kdDebug() << "kio_kiobashinProtocol::~kio_kiobashinProtocol()" << endl;
}

static void app_entry(UDSEntry& e, unsigned int uds, const QString& str)
{
  UDSAtom a;
  a.m_uds = uds;
  a.m_str = str;
  e.append(a);
}

// appends an int with the UDS-ID uds
static void app_entry(UDSEntry& e, unsigned int uds, long l)
{
  UDSAtom a;
  a.m_uds = uds;
  a.m_long = l;
  e.append(a);
}

static void app_dir(UDSEntry& e, const QString & n, size_t s)
{
        e.clear();
        app_entry(e, KIO::UDS_NAME, QFile::decodeName(n.local8Bit()));
        app_entry(e, KIO::UDS_FILE_TYPE, S_IFDIR);
        app_entry(e, KIO::UDS_ACCESS, 0400);
        app_entry(e, KIO::UDS_SIZE, s);
        app_entry(e, KIO::UDS_MIME_TYPE, "inode/directory");
}

static void app_file(UDSEntry& e, const QString & n, size_t s,const QString & mime)
{
        e.clear();
        app_entry(e, KIO::UDS_NAME, QFile::decodeName(n.local8Bit()));
        app_entry(e, KIO::UDS_FILE_TYPE, S_IFREG);
        app_entry(e, KIO::UDS_ACCESS, 0400);
        app_entry(e, KIO::UDS_SIZE, s);
        app_entry(e, KIO::UDS_MIME_TYPE, mime);
}

static QString runscript (const QString & synt) {
	int count=0;
	QString data;
	FILE *fd = popen(synt, "r");
	char buffer[ 4090 ];
	QByteArray array;

	while ( !feof( fd ) )
	{
	int n = fread( buffer, 1, 2048, fd );
	if ( n == -1 )
	{
	// ERROR
	pclose( fd );
	break;
	}
	array.setRawData( buffer, n );
	data.append(array);	
	array.resetRawData( buffer, n );
	}
	pclose( fd );
	return data;
}

static QString pathtosyntax (const QString & path, const QString & action, const QString & url) {
	QStringList synthax=QStringList::split("/",url);
	QString exec;
	int count=0;
	exec.append("/bin/bash "+path);
	for ( QStringList::Iterator it = synthax.begin(); it != synthax.end(); ++it ) {
		if (count==0) {
			exec.append(*it);
			exec.append(" \""+action+"\" ");
		} else {
			exec.append("\"");
			exec.append(*it);
			exec.append("\" ");
		}
		count++;
	}
	return exec;
}

void kio_kiobashinProtocol::get(const KURL& url )
{

	QString exec=pathtosyntax(QDir::homeDirPath()+"/kiobashin/","*mime",url.path());
	QString mimet=runscript(exec);

	mimeType(mimet);

	exec=pathtosyntax(QDir::homeDirPath()+"/kiobashin/","*file",url.path());

	FILE *fd = popen(exec, "r");
	char buffer[ 4090 ];
	QByteArray array;

	while ( !feof( fd ) )
	{
	int n = fread( buffer, 1, 2048, fd );
	if ( n == -1 )
	{
	// ERROR
	pclose( fd );
	break;
	}
	array.setRawData( buffer, n );
	data(array);
	array.resetRawData( buffer, n );
	}
	pclose( fd );
	finished();

/*
    kdDebug() << "kio_kiobashin::get(const KURL& url)" << endl ;
    
    kdDebug() << "Seconds: " << url.query() << endl;
    QString remoteServer = url.host();
    int remotePort = url.port();
    kdDebug() << "myURL: " << url.prettyURL() << endl;
    
    infoMessage(i18n("Looking for %1...").arg( remoteServer ) );
    // Send the mimeType as soon as it is known


    mimeType("text/html");
    // Send the data
    QString theData = i18n("This is a contenntntntn test of kio_kiobashin %1").arg( url.path() );
    data(QCString(theData.local8Bit()));
    data(QByteArray()); // empty array means we're done sending the data
    finished();
*/
}

void kio_kiobashinProtocol::stat(const KURL & url)
{
	UDSEntry entry;	
	UDSAtom atom;

	QStringList syntax=QStringList::split("/",url.path());

	atom.m_uds = KIO::UDS_NAME;
	atom.m_str = syntax.last();
	entry.append(atom);
	
	atom.m_uds = KIO::UDS_FILE_TYPE;
	atom.m_long = S_IFREG;
	entry.append(atom);
	
	atom.m_uds = KIO::UDS_ACCESS;
	atom.m_long = 0666;
	entry.append(atom);
	
	atom.m_uds = KIO::UDS_SIZE;
	atom.m_long = 0;
	entry.append(atom);

	statEntry(entry);
	
	finished();
}

void kio_kiobashinProtocol::listDir(const KURL & url)
{
	UDSEntry entry;
	int count=0;

	if (url.path()=="/") {
		/*
			Show avaible scripts as folders
		*/
		QDir d(QDir::homeDirPath()+"/kiobashin","*.sh");
		d.setFilter( QDir::Files | QDir::Hidden | QDir::NoSymLinks );
		for ( int i = 0; i < d.count(); i++ ) {
			app_dir(entry, d[i], 0);
			listEntry(entry, false);
			count++;
		}
	} else {
		/*
		Prepare syntax
		*/
		QString exec=pathtosyntax(QDir::homeDirPath()+"/kiobashin/","*dir",url.path());
		/*
		Executes script
		*/
		QString data=runscript(exec);
	
		/*
		Return directories
		*/
		QStringList items=QStringList::split("\n",data);
		for ( QStringList::Iterator it = items.begin(); it != items.end(); ++it ) {
			QString name=*it;
			if (name.left(1)=="*") {
				app_dir(entry, name.mid(1), 0);
				listEntry(entry, false);
			}
			if (name.left(1)==".") {
				++it;
				QStringList arguments=QStringList::split("*",*it);
				QString size=*arguments.at(0);
				app_file(entry, name.mid(1), size.toInt(),*arguments.at(1));
				listEntry(entry, false);
			}
	
			count++;
		}

	}
	
	totalSize(count);
	listEntry(entry, true);
	finished();

}

extern "C"
{
    int kdemain(int argc, char **argv)
    {
        KInstance instance( "kio_kiobashin" );
        
        kdDebug(7101) << "*** Starting kio_kiobashin " << endl;
        
        if (argc != 4) {
            kdDebug(7101) << "Usage: kio_kiobashin  protocol domain-socket1 domain-socket2" << endl;
            exit(-1);
        }
        
        kio_kiobashinProtocol slave(argv[2], argv[3]);
        slave.dispatchLoop();
        
        kdDebug(7101) << "*** kio_kiobashin Done" << endl;
        return 0;
    }
} 
